package main

import (
	"fmt"
	"io/ioutil"
	"os"
	"path/filepath"
	"strconv"
)

const (
	tokenFilePath   = ".config/deepin/deepin-sync-daemon/token"
	machineFilePath = ".config/deepin/deepin-sync-daemon/machine_data"

	lastSyncDir      = ".cache/deepin/deepin-sync-daemon/local"
	lastSyncBasename = "last_sync_time"
)

func loadMachineData() ([]byte, error) {
	filename := filepath.Join(os.Getenv("HOME"), machineFilePath)
	return ioutil.ReadFile(filename)
}

func saveMachineData(data []byte) error {
	filename := filepath.Join(os.Getenv("HOME"), machineFilePath)
	err := ensureParentDirExists(filename)
	if err != nil {
		return err
	}
	return ioutil.WriteFile(filename, data, 0644)
}

func getLastSyncTime(uid int64) (int64, error) {
	if uid == 0 {
		return 0, nil
	}
	filename := filepath.Join(os.Getenv("HOME"), lastSyncDir,
		fmt.Sprint(uid), lastSyncBasename)
	contents, err := ioutil.ReadFile(filename)
	if err != nil {
		return 0, err
	}
	return strconv.ParseInt(string(contents), 10, 64)
}

func saveLastSyncTime(uid, timestamp int64) error {
	if uid == 0 {
		return nil
	}
	filename := filepath.Join(os.Getenv("HOME"), lastSyncDir,
		fmt.Sprint(uid), lastSyncBasename)
	err := ensureParentDirExists(filename)
	if err != nil {
		return err
	}
	return ioutil.WriteFile(filename,
		[]byte(fmt.Sprint(timestamp)), 0644)
}

func ensureParentDirExists(filename string) error {
	return os.MkdirAll(filepath.Dir(filename), 0755)
}
