package hardware

import (
	"encoding/json"
)

type Hardware struct {
	ID        string `json:"id"`
	Hostname  string `json:"hostname"`
	Username  string `json:"username"`
	OS        string `json:"os"`
	CPU       string `json:"cpu"`
	Laptop    bool   `json:"laptop"`
	Memory    int64  `json:"memory"`
	DiskTotal int64  `json:"disk_total"` // only for root disk
	DMI       *DMI   `json:"dmi"`
}

const (
	fileHostname = "/etc/hostname"
)

func GetHardware() (*Hardware, error) {
	os, err := getOSFromDeepin(fileOS)
	if err != nil {
		return nil, err
	}
	hostname, err := readFile(fileHostname)
	if err != nil {
		return nil, err
	}
	cpu, _ := getKeyFromCPUFile(cpuKeyModelName, cpuProcFile)
	mem, _ := getKeyFromMemFile(memKeyTotal, memProcFile)
	laptop, _ := hasBattery(batterySysfsDir)
	total, _ := getDiskTotal()
	d, err := GetDMI()
	if err != nil {
		return nil, err
	}
	return &Hardware{
		ID:        d.ID(),
		Hostname:  hostname,
		OS:        os,
		CPU:       cpu,
		Laptop:    laptop,
		Memory:    mem,
		DiskTotal: total,
		DMI:       d,
	}, nil
}

func (hw *Hardware) Bytes() []byte {
	data, _ := json.Marshal(hw)
	return data
}
