package hardware

import (
	"testing"
)

func TestCPU(t *testing.T) {
	v, err := getKeyFromCPUFile(cpuKeyModelName, "testdata/cpuinfo")
	if err != nil {
		panic(err)
	}
	if v != "Intel(R) Core(TM) i7-3520M CPU @ 2.90GHz" {
		panic("Function wrong")
	}
}

func TestMemory(t *testing.T) {
	v, err := getKeyFromMemFile(memKeyTotal, "testdata/meminfo")
	if err != nil {
		panic(err)
	}
	if v != 7857260 {
		panic("Function wrong")
	}
}

func TestOS(t *testing.T) {
	v, err := getOSFromDeepin("testdata/deepin-version")
	if err != nil {
		panic(err)
	}
	if v != "Deepin 15.8" {
		panic("Function wrong")
	}

	v, err = getOSFromDeepin("testdata/deepin-version-professional")
	if err != nil {
		panic(err)
	}
	if v != "Deepin Professional 15.8" {
		panic("Function wrong")
	}
}

func TestDMI(t *testing.T) {
	dmi, err := doGetDMI("testdata/dmi")
	if err != nil {
		panic(err)
	}
	if dmi.ID() != "c117a6fec2e12ef1f7aaff09532b7d4946d39d1f6af2ad1ea91dd15a83a2ab50" {
		panic("Function wrong")
	}
	if dmi.ProductFamily != "ThinkPad X230" {
		panic("Function wrong")
	}
}

func TestBattery(t *testing.T) {
	has, err := hasBattery("testdata/power_supply")
	if err != nil {
		panic(err)
	}
	if !has {
		panic("Function wrong")
	}

	has, err = hasBattery("testdata/power_supply_ac")
	if err != nil {
		panic(err)
	}
	if has {
		panic("Function wrong")
	}
}

func TestDisk(t *testing.T) {
	var data = `NAME      MAJ:MIN RM         SIZE RO TYPE MOUNTPOINT
/dev/sda    8:0    0 500107862016  0 disk
/dev/sda1   8:1    0   4294967296  0 part /boot
/dev/sda2   8:2    0   4294967296  0 part [SWAP]
/dev/sda3   8:3    0         1024  0 part
/dev/sda5   8:5    0 107374182400  0 part /Data
/dev/sda6   8:6    0  53687091200  0 part /
/dev/sda7   8:7    0  32212254720  0 part
/dev/sda8   8:8    0  30719082496  0 part`
	dlist, err := decodeLsblkOutputs([]byte(data))
	if err != nil {
		panic(err)
	}
	if len(dlist) != 1 || len(dlist[0].PartList) != 7 {
		panic("Decode error")
	}

	if dlist[0].Size != 500107862016 {
		panic("Decode error")
	}
}
