package hardware

import (
	"bufio"
	"fmt"
	"os"
	"strconv"
	"strings"
)

const (
	procFileMode = os.FileMode(0444)

	cpuProcFile     = "/proc/cpuinfo"
	cpuKeyModelName = "model name"

	memProcFile = "/proc/meminfo"
	memKeyTotal = "MemTotal"
)

func getKeyFromCPUFile(key, filename string) (string, error) {
	fr, err := os.OpenFile(filename, os.O_RDONLY, procFileMode)
	if err != nil {
		return "", err
	}
	defer fr.Close()

	var scanner = bufio.NewScanner(fr)
	value := scanProcFile(key+"\t", scanner)
	if len(value) == 0 {
		return "", fmt.Errorf("not found key: %s", key)
	}
	return strings.TrimSpace(value), nil
}

func getKeyFromMemFile(key, filename string) (int64, error) {
	fr, err := os.OpenFile(filename, os.O_RDONLY, procFileMode)
	if err != nil {
		return 0, err
	}
	defer fr.Close()

	var scanner = bufio.NewScanner(fr)
	value := scanProcFile(key, scanner)
	if len(value) == 0 {
		return 0, fmt.Errorf("not found key: %s", key)
	}
	value = strings.TrimSpace(value)
	value = strings.TrimRight(value, " kB")
	return strconv.ParseInt(value, 10, 64)
}

func scanProcFile(key string, scanner *bufio.Scanner) string {
	for scanner.Scan() {
		var line = scanner.Text()
		if len(line) == 0 {
			continue
		}
		if !strings.Contains(line, key) {
			continue
		}
		list := strings.Split(line, ":")
		if len(list) < 2 {
			continue
		}
		return list[1]
	}
	return ""
}
