package utils

import (
	"errors"
	"github.com/fsnotify/fsnotify"
)

type FSWatcher struct {
	watcher *fsnotify.Watcher
	files   []string
}

var (
	ErrFSWatchInvalidEvent = errors.New("invalid fsnotify event")
)

func NewFSWatcher(files []string) (*FSWatcher, error) {
	watcher, err := fsnotify.NewWatcher()
	if err != nil {
		return nil, err
	}
	for _, f := range files {
		err := watcher.Add(f)
		if err != nil {
			watcher.Close()
			return nil, err
		}
	}
	return &FSWatcher{
		watcher: watcher,
		files:   files,
	}, nil
}

func (fw *FSWatcher) Close() {
	if fw.watcher != nil {
		fw.watcher.Close()
	}
}

func (fw *FSWatcher) Watch(eventHandler func(fsnotify.Event),
	errorHandler func(error)) error {
	for {
		select {
		case ev, ok := <-fw.watcher.Events:
			if !ok {
				return ErrFSWatchInvalidEvent
			}
			eventHandler(ev)
		case err, ok := <-fw.watcher.Errors:
			if !ok {
				return ErrFSWatchInvalidEvent
			}
			errorHandler(err)
			return err
		}
	}
}
