package modules

import (
	"encoding/json"
	"pkg.deepin.io/daemon/sync/modules/appearance"
	"pkg.deepin.io/daemon/sync/modules/background"
)

type appearanceWrapper struct {
	appearance.Data
	BackgroundURIs    []string `json:"background_uris"`
	GreeterBackground string   `json:"greeter_background"`
}

func (wrapper *appearanceWrapper) ToAppearanceData() ([]byte, error) {
	return json.Marshal(&wrapper.Data)
}

func (wrapper *appearanceWrapper) ToBackgroundData() ([]byte, error) {
	info := background.NewData(wrapper.Version,
		wrapper.BackgroundURIs, wrapper.GreeterBackground)
	return json.Marshal(info)
}

func (wrapper *appearanceWrapper) FixAppearance(data []byte) error {
	var info appearance.Data
	err := json.Unmarshal(data, &info)
	if err != nil {
		return err
	}
	wrapper.Data = info
	return nil
}

func (wrapper *appearanceWrapper) FixBackground(data []byte) error {
	var info background.Data
	err := json.Unmarshal(data, &info)
	if err != nil {
		return err
	}
	if info.Workspace != nil {
		wrapper.BackgroundURIs = info.Workspace.Files()
	}
	if info.Greeter != nil {
		wrapper.GreeterBackground = info.Greeter.URI
	}
	return nil
}

func (wrapper *appearanceWrapper) Bytes() []byte {
	data, _ := json.Marshal(wrapper)
	return data
}
